package com.heyang;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

public class Lexer {
	private List<Token> tokens;
	
	public Lexer(String inputTxt) {
		tokens = new ArrayList<Token>();

		String bundle = "";
		for (int i = 0; i < inputTxt.length(); i++) {
			char c = inputTxt.charAt(i);

			if (Character.isWhitespace(c)) {
				if (StringUtils.isNotEmpty(bundle.trim())) {
					addText2Tokens(bundle);
					bundle = "";
				}
				
				continue;
			} else if (c == '<') {
				int next=i+1;
            	if(next<inputTxt.length() && inputTxt.charAt(next)=='/') {
            		addText2Tokens(bundle);
            		bundle="";
            		tokens.add(new Token("</",Token.TYPE_END_OPEN));
            		i++;
            	}else {
            		tokens.add(new Token(c, Token.TYPE_OPEN_ANGLEBRACKET));
            	}
				
			} else if (c == '>') {
				if (StringUtils.isNotEmpty(bundle)) {
					addText2Tokens(bundle);
					bundle = "";
				}

				tokens.add(new Token(c, Token.TYPE_CLOSE_ANGLEBRACKET));
			}else if (c == '=') {
				if (StringUtils.isNotEmpty(bundle)) {
					addText2Tokens(bundle);
					bundle = "";
				}

				tokens.add(new Token(c, Token.TYPE_EQUAL));
			}  else if (c == '/') {
				int next=i+1;
            	if(next<inputTxt.length() && inputTxt.charAt(next)=='>') {
            		addText2Tokens(bundle);
            		bundle="";
            		tokens.add(new Token("/>",Token.TYPE_EMPTY_CLOSE));
            		i++;
            	}else {
            		tokens.add(new Token(c, Token.TYPE_slant));
            	}
				
			} else if(c == '\"') {
				int idx=i+1;
				
				while(idx<inputTxt.length()) {
					char cEnd = inputTxt.charAt(idx);
					
					if (cEnd == '\"') {
						break;
					}
					
					idx++;
				}
				
				String sub=inputTxt.substring(i, idx+1);
				tokens.add(new Token(sub, Token.TYPE_TEXT));
				i=idx;
			} else {
				bundle += c;
			}
		}
		
		setTokenIndexes();
	}
	
	private boolean addText2Tokens(String text) {
		if(StringUtils.isNotEmpty(text)) {
			tokens.add(new Token(text, Token.TYPE_TEXT));
			return true;
		}else {
			return false;
		}
	}

	public void setTokenIndexes() {
		int idx = 0;
		for (Token t : tokens) {
			idx++;
			t.setIndex(idx);
		}
	}

	public void printTokens() {
		int idx = 0;
		for (Token t : tokens) {
			idx++;
			t.setIndex(idx);
			System.out.println("#" + idx + " " + t.getText());
		}
	}
	
	public String getCompactJsonTxt() {
		StringBuilder sb=new StringBuilder();
		
		for (Token t : tokens) {
			sb.append(t.getText()+" ");
		}
		
		return sb.toString();
	}
	
	public List<Token> getTokens() {
		return tokens;
	}
}
