package com.heyang;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class Node implements Comparable<Node>{
	private String text;
	private String name;
	private List<Node> children;
	private List<Property> proterties;
    private int depth=0;
    
    public int compareTo(Node another) {
        return this.name.compareTo(another.name);
    }
	
	public void addChild(Node n) {
		if(children==null) {
			children=new ArrayList<Node>();
		}
		
		children.add(n);
		adjustDepth();
	}
	
	private void adjustDepth() {
		if(children==null) {
			return;
		}
        for(Node json:children) {
            json.depth=this.depth+1;
            json.adjustDepth();
        }
    }
	
	public void addProperty(Property p) {
		if(proterties==null) {
			proterties=new ArrayList<Property>();
		}
		
		proterties.add(p);
	}
	
	public String toString() {
		String tabs=getIndentSpace();
		
        StringBuilder sb=new StringBuilder();
        sb.append(tabs);
        
        sb.append("<"+name);
        if(proterties!=null) {
        	Collections.sort(proterties);
        	
        	for(Property p:proterties) {
        		sb.append(" "+p.getName()+"="+p.getValue());
        	}
        }
        
        if(text==null && children==null) {
        	sb.append("/>");
        	return sb.toString();
        }else {
        	sb.append(">");
        }
        
        if(text!=null) {
        	sb.append(text);
        }
        
        if(children!=null) {
        	
        	
        	Collections.sort(children);
        	for(Node child:children) {
        		sb.append("\n");
        		sb.append(child);
        	}
        }
        
        if(children!=null) {
        	sb.append("\n"+tabs+"</"+name+">");
        }else {
        	sb.append("</"+name+">");
        }
        
        return sb.toString();
	}
	
	private String getIndentSpace() {
        return String.join("", Collections.nCopies(this.depth, "    "));
    }
	
	public String getText() {
		return text;
	}
	public void setText(String text) {
		this.text = text;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public List<Node> getChildren() {
		return children;
	}
	public void setChildren(List<Node> children) {
		this.children = children;
	}
	public List<Property> getProterties() {
		return proterties;
	}
	public void setProterties(List<Property> proterties) {
		this.proterties = proterties;
	}
}
